# Generated by roxygen2: do not edit by hand

S3method("+",raw)
S3method("+",raw_bytes)
S3method(future::resolved,RequestFuture)
export(add_user_to_chat)
export(batch_download_channels)
export(check_invite_link)
export(check_phone_on_telegram)
export(check_phones_on_telegram)
export(check_username_on_telegram)
export(check_usernames_on_telegram)
export(download_channel_info)
export(download_channel_media)
export(download_channel_members)
export(download_channel_messages)
export(download_channel_reactions)
export(download_channel_replies)
export(estimate_channel_post_count)
export(get_full_user)
export(increase_view_count)
export(join_channel)
export(join_private_chat)
export(join_public_channel)
export(send_audio)
export(send_document)
export(send_file)
export(send_message)
export(send_photo)
export(send_video)
export(update_profile)
export(update_profile_photo)
export(update_username)
importFrom(httr,GET)
importFrom(httr,content)
importFrom(logger,log_debug)
importFrom(logger,log_info)
importFrom(logger,log_warn)
importFrom(mime,mimemap)
importFrom(stats,setNames)
importFrom(xml2,read_xml)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_children)
importFrom(xml2,xml_name)
importFrom(xml2,xml_text)
importFrom(xml2,xml_type)
useDynLib(telegramR, .registration = TRUE)
