# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

aes_ige_encrypt <- function(key_raw, iv_raw, data_raw) {
    .Call(`_telegramR_aes_ige_encrypt`, key_raw, iv_raw, data_raw)
}

aes_ige_decrypt <- function(key_raw, iv_raw, data_raw) {
    .Call(`_telegramR_aes_ige_decrypt`, key_raw, iv_raw, data_raw)
}

pack_int64 <- function(x) {
    .Call(`_telegramR_pack_int64`, x)
}

unpack_int64 <- function(raw_vector) {
    .Call(`_telegramR_unpack_int64`, raw_vector)
}

xor_bytes_cpp <- function(a, b) {
    .Call(`_telegramR_xor_bytes`, a, b)
}

bytes_to_int32_cpp <- function(bytes, is_signed) {
    .Call(`_telegramR_bytes_to_int32`, bytes, is_signed)
}

bytes_to_hex_be_cpp <- function(bytes) {
    .Call(`_telegramR_bytes_to_hex_be`, bytes)
}

serialize_bytes_cpp <- function(data) {
    .Call(`_telegramR_serialize_bytes_cpp`, data)
}

pack_int32_cpp <- function(value) {
    .Call(`_telegramR_pack_int32`, value)
}

hex_to_bytes_cpp <- function(hex_str, out_len, little_endian) {
    .Call(`_telegramR_hex_to_bytes`, hex_str, out_len, little_endian)
}

encode_waveform_cpp <- function(waveform) {
    .Call(`_telegramR_encode_waveform_cpp`, waveform)
}

decode_waveform_cpp <- function(waveform) {
    .Call(`_telegramR_decode_waveform_cpp`, waveform)
}

